# -*- coding: utf8 -*-

import os
from .utils import makedirs, platformType

class Config:

    BASE_PATH = os.path.expanduser('~')
    GAME_ROOT_DIR = os.path.join(BASE_PATH,'.minecraft')
    GAME_LIB_DIR = os.path.join(GAME_ROOT_DIR,'libraries')
    GAME_VERSION_DIR = os.path.join(GAME_ROOT_DIR,'versions')
    GAME_ASSET_DIR = os.path.join(GAME_ROOT_DIR,'assets')
    GAME_DEPLOY_DIR = os.path.join(GAME_ROOT_DIR, 'deploy')
    GAME_SPIGOT_DEPLOY_DIR = os.path.join(GAME_ROOT_DIR, 'spigot')

    def __init__(self,version, isSpigot = False):
        self.version=version
        self.isSpigot = isSpigot


    def version_json_path(self):
        '''Game Config JSON File Path'''
        return os.path.join(self.versionDir(),self.version+'.json')    

    # Client
    def assets_indexes_dir(self):
        '''Client Assets Index JSON File Directory'''
        dir = os.path.join(Config.GAME_ASSET_DIR,'indexes')
        makedirs(dir)
        return dir
    
    def assets_objects_dir(self, hash):
        '''Client Assets Object Directory'''
        dir = os.path.join(Config.GAME_ASSET_DIR,'objects',hash[0:2])
        makedirs(dir)
        return dir
    
    def versionDir(self):
        '''Client Version Related Directory'''
        dir = os.path.join(Config.GAME_VERSION_DIR,self.version)
        makedirs(dir)
        return dir

    def client_jar_path(self):
        '''Client Game JAR File Path'''
        return os.path.join(Config.GAME_VERSION_DIR,self.version,'client.jar')
    

    def client_library_dir(self, subpath = None):
        '''Client Dependiencies Libraries Directory'''
        dir = Config.GAME_LIB_DIR
        if None != subpath:
            subdir =  os.path.dirname(subpath)
            dir = os.path.join(dir,subdir)
        makedirs(dir)
        return dir
        
    def client_native_dir(self):
        '''Client Native Related dependencies Directory'''
        dir = os.path.join(self.GAME_VERSION_DIR, self.version, self.version + '-native')
        makedirs(dir)
        return dir

    # Server
    def server_jar_path(self):
        '''Server Game JAR File Path'''
        return os.path.join(Config.GAME_VERSION_DIR,self.version)

    def server_deploy_path(self):
        '''Server Deploy Path'''
        deployPath = Config.GAME_SPIGOT_DEPLOY_DIR if self.isSpigot else Config.GAME_DEPLOY_DIR
        makedirs(deployPath)
        return deployPath

    def eula_path(self):
        return os.path.join(self.server_deploy_path(), 'eula.txt')

    def properties_path(self):
        return os.path.join(self.server_deploy_path(), 'server.properties')
    
    def server_deploy_build_path(self):
        deployBuildPath = os.path.join(self.server_deploy_path(), 'build')
        makedirs(deployBuildPath)
        return deployBuildPath

    def server_spigot_jar_path(self, isInBuildDir=False):
        return os.path.join(self.server_deploy_build_path() if isInBuildDir else self.server_deploy_path(), 'spigot-' + self.version + '.jar')

    def server_craftbukkit_jar_path(self, isInBuildDir=False):
        return os.path.join(self.server_deploy_build_path() if isInBuildDir else self.server_deploy_path(), 'craftbukkit-' + self.version + '.jar')



# -*- coding: utf8 -*-

from .Config import Config
import requests
import json
import os
from .utils import makedirs

class Mojang:

    version_list_url = 'https://launchermeta.mojang.com/mc/game/version_manifest.json'
    asset_base_url = 'https://resources.download.minecraft.net/'

    versions = None

    @classmethod
    def get_version_list(cls, update=False):
        '''Get All Version Game Configuration'''
        makedirs(Config.GAME_ROOT_DIR)
        localFilePath = os.path.join(Config.GAME_ROOT_DIR,os.path.basename(Mojang.version_list_url))
        resp = None
        if (not update) and os.path.exists(localFilePath):
            with open(localFilePath,'r') as localFile:
                resp = json.load(localFile)
                print('Use Local File For Game Version Manifest JSON File')
        else:
            resp = json.loads(requests.get(Mojang.version_list_url).text)
            with open(localFilePath,'w') as localFile:
                json.dump(resp,localFile)
                print('Download Game Version Manifest JSON File from Mojang server and cached')


        Mojang.versions = resp.get('versions')    
        return Mojang.versions

    @classmethod
    def get_release_version_list(cls):
        '''Get Game Release Version List'''
        versions = Mojang.get_version_list()
        release = list(filter(lambda version: version.get('type') == 'release', versions))
        return release


    @classmethod
    def get_release_game_json(cls, id):
        releases = list(filter(lambda release: release.get('id') == id, Mojang.get_release_version_list()))
        if len(releases) > 0 : 
            url = releases[0].get('url')
            hash = os.path.split(os.path.dirname(url))[-1]
            return (url, hash)
        else:
            return None
        
    @classmethod
    def assets_objects_url(cls,hash):
        return os.path.join(Mojang.asset_base_url,hash[0:2],hash)
    


# -*- coding: utf8 -*-

from .GameDownloader import GameDownloader
import sys, getopt
from .Mojang import Mojang
from .utils import hint, ColorString

version = ""
username = "guest"

def showVersionList(isShow = True, isServer = False):
    
    global version

    if not isShow: return

    allVersions = Mojang.get_version_list(update=True)
    releases = list(filter(lambda version: version.get('type') == 'release', allVersions))
    releaseVersions = list(map(lambda info: info.get('id', ''), releases))
    print(ColorString.warn('\nAll Release Versions as follow:\n'))
    tabSpace = '\t'
    lineLeadingSpace = '  '
    versionInfo = lineLeadingSpace
    for index, releaseVersion in enumerate(releaseVersions): 
        versionInfo = versionInfo + str(releaseVersion)
        if (index + 1) % 9 == 0:
            versionInfo = versionInfo + '\n' + lineLeadingSpace
        else:
            versionInfo = versionInfo + tabSpace
    print(ColorString.string(versionInfo,ColorString.FG_GREEN,displayMode=ColorString.HIGHLIGHT))

    if len(releaseVersions) > 0:
        version = releaseVersions[0]

    select = hint(ColorString.warn('\nPlease select a version number of above list to %s %s ') % ('deploy' if isServer else 'play' , ColorString.error('(default: %s):' % version)))
    if len(select) > 0:
        found = False
        for releaseVersion in releaseVersions: 
            if releaseVersion == select.strip():
                found = True
                version = releaseVersion
                print(ColorString.confirm('You choose the version: %s') % version)
        if not found:
            print(ColorString.warn('There is no such a release version game, use default!'))
    else:
        print(ColorString.confirm('You choose the default version(%s)!') % version)

def showUserName(isShow):

    if not isShow: return 

    global username

    u = hint(ColorString.warn('Please choose a username %s ') % ColorString.error('(default: %s):' % username))
    if len(u) > 0:
        username = u
        print(ColorString.confirm('You username in game is: %s') % username)
    else:
        print(ColorString.warn('Use the default username!!!'))


def startClient():

    isShowList = True
    isShowUserName = True

    global version
    global username

    try:
        opts, _ = getopt.getopt(sys.argv[1:], "v:u:h", ["version=", "username=","help"])
        for o, a in opts:
            if o in ["-v", "--version"]:
                if len(a) > 0:
                    version = a
                    isShowList = False
            if o in ["-u", "--username"]:
                if len(a) > 0:
                    username = a
                    isShowUserName = False
            if o in ["-h", "--help"]:
                help()

        showVersionList(isShowList)
        showUserName(isShowUserName)
        game = GameDownloader(version)
        game.downloadGameJSON()
        game.downloadClient()
        game.downloadAssetIndex()
        game.downloadAssetObjects()
        game.donwloadLibraries()
        game.startCient(user=username)

    except getopt.GetoptError:
        print("The arguments is invalid!")

def downloadServer():
    
    global version

    mem_start = ""
    mem_max = ""
    isShowList = True
    isSpigotServer = False

    try:

        opts, _ = getopt.getopt(sys.argv[1:], "v:s:x:ho", ["version=", "mem_start=", "mem_max=","help", "spigot"])
        for o, a in opts:
            if o in ["-v", "--version"]:
                if len(a) > 0:
                    version = a
                    isShowList = False
            if o in ["-s", "--mem_start"]:
                if len(a) > 0:
                    mem_start = a
            if o in ["-x", "--mem_max"]:
                if len(a) > 0:
                    mem_max = a               
            if o in ["-h", "--help"]:
                help(False)
            if o in ["-o", "--spigot"]:
                isSpigotServer = True


        showVersionList(isShowList, isServer=True)
        game = GameDownloader(version, isSpigot=isSpigotServer)
        if not isSpigotServer:
            game.downloadGameJSON()
            game.downloadServer()

        if len(mem_start) > 0 and len(mem_max) > 0 :
            game.deployServer(mem_start=mem_start, mem_max=mem_max)
        elif len(mem_start) > 0:
            game.deployServer(mem_start=mem_start)
        elif len(mem_max) > 0:
            game.deployServer(mem_max=mem_max)
        else :
            game.deployServer()

    except getopt.GetoptError:
        print("The arguments is invalid!") 


def help(isClient = True):

    if isClient:
        print("""
        NAME

            orzmc -- A command for start minecraft client

        Usage

            orzmc [-v client_version_number] [-u username] [-h]

                -v, --version  
                    Specified the Minecraft clinet version number to start

                -u, --username 
                    pick an username for player when start the client

                -h, --help 
                    show the client command usage info

        """)
    else:
        print("""
        NAME

            orzmcs -- A tool for deploy minecraft server

        Usage: 
           
            orzmcs [-v server_version_number] [-s memory_start] [-x memory_max] [-ho]

                -v, --version 
                    Specified the Minecraft server version number to deploy
                
                -s, --mem_start 
                    Specified the JVM initial memory allocation
                
                -x, --mem_max
                    Specified the JVM max memory allocation
                
                -h, --help
                    show the server command usage info
                
                -o, --spigot
                    deploy spigot server for performance boost

        """)
    exit(0)


# -*- coding: utf8 -*-

from .Mojang import Mojang
from .Config import Config
from .Spigot import Spigot
from .utils import checkFileExist, isPy3, platformType, ColorString
import json
import requests
import os
import sys
import signal
import uuid
import re
import hashlib
import time
import progressbar
import io
import shutil

is_sigint_up = False
def sigint_handler(signum, frame):
    global is_sigint_up
    is_sigint_up = True
    print(ColorString.warn("\nForce Exit!"))
    exit(-1)

signal.signal(signal.SIGINT, sigint_handler)
signal.signal(signal.SIGHUP, sigint_handler)
signal.signal(signal.SIGTERM, sigint_handler)


class GameDownloader:

    def __init__(self, version, isSpigot = False):
        self._game=None
        self._assets=None
        self._javaClassPathList = None
        self.config = Config(version, isSpigot=isSpigot)

    def download(self, url, dir):
        global is_sigint_up
        if is_sigint_up:
            return
        
        filename = os.path.basename(url)
        with open(os.path.join(dir,filename),'wb') as f:
            f.write(requests.get(url).content)

    def loadJSON(self, filePath):
        with open(filePath) as json_data:
            jsonObj = json.load(json_data)
            return jsonObj

    def downloadGameJSON(self):
        global is_sigint_up
        if is_sigint_up:
            return

        '''Download Game Json Configure File'''
        version_json_path = self.config.version_json_path()
        (url, hash) = Mojang.get_release_game_json(self.config.version)
        if not checkFileExist(version_json_path, hash):
            print("Download Game Json Configure File!")
            jsonStr = requests.get(url).text
            if jsonStr != None:
                if isPy3:
                    with open(version_json_path,'w',encoding='utf-8') as f:
                        f.write(jsonStr)
                else:
                    with open(version_json_path,'w') as f:
                        f.write(jsonStr.encode('utf-8'))
        else: 
            print("Game Json Configure File have been downloaded!")

    def game(self):
        if self._game == None:
            self._game = self.loadJSON(self.config.version_json_path())
        return self._game
    
    def assets(self):
        if self._assets == None:
            index_json_url = self.game().get('assetIndex').get('url')
            index_json_path = os.path.join(self.config.assets_indexes_dir(), os.path.basename(index_json_url))
            self._assets = self.loadJSON(index_json_path)
        return self._assets

    def javaClassPathList(self):
        if None == self._javaClassPathList:
            self._javaClassPathList = []
            libs = self.game().get('libraries')
            total = len(libs)
            
            errorMsg = []
            with progressbar.ProgressBar(max_value=total, prefix='javaClassPathList: '):
                for lib in libs: 
                    libName = lib.get('name')
                    downloads = lib.get('downloads')

                    rules = lib.get('rules')
                    if None != rules:
                        for rule in rules:
                            if None != rule:
                                if rule.get('action') == 'disallow':
                                    if rule.get('os').get('name') == platformType():
                                        errorMsg.append(libName + 'is disallowed')
                                        continue

                    libPath = None
                    url = None
                    sha1 = None
                    nativeKey = 'natives-'+ platformType()
                    if 'natives' in lib:
                        platform = lib.get('natives').get(platformType())
                        if platform == None:
                            errorMsg.append('Error: no platform jar - ' +  libName)
                            continue
                        else:
                            libPath = downloads.get('classifiers').get(platform).get('path')
                            url = downloads.get('classifiers').get(platform).get('url')
                            sha1 = downloads.get('classifiers').get(platform).get('sha1')
                            nativeFilePath = os.path.join(self.config.client_native_dir(),os.path.basename(url))
                            if not checkFileExist(nativeFilePath,sha1):
                                errorMsg.append("Not Exist: %s" % nativeFilePath)
                                continue
                            else:
                                self._javaClassPathList.append(nativeFilePath)
                    else:
                        classifiers = downloads.get('classifiers')
                        if classifiers and nativeKey in downloads.get('classifiers'):
                            url = downloads.get('classifiers').get(nativeKey).get('url')
                            sha1 = downloads.get('classifiers').get(platform).get('sha1')
                            nativeFilePath = os.path.join(self.config.client_native_dir(),os.path.basename(url))
                            if not checkFileExist(nativeFilePath,sha1):
                                errorMsg.append("Not Exist: %s" % nativeFilePath)
                                continue
                            else:
                                self._javaClassPathList.append(nativeFilePath)
                                
                        libPath = downloads.get('artifact').get('path')
                        url = downloads.get('artifact').get('url')
                        sha1 = downloads.get('artifact').get('sha1')
                        libFilePath = os.path.join(self.config.client_library_dir(), libPath)
                        if not checkFileExist(libFilePath,sha1):
                            errorMsg.append("Not Exist: %s" % libFilePath)
                            continue            
                        else:
                            self._javaClassPathList.append(libFilePath)
            if(len(errorMsg) > 0):
                print('\n'.join(errorMsg))
            self._javaClassPathList.append(self.config.client_jar_path())
        return self._javaClassPathList

# Assets

    def downloadAssetIndex(self):

        global is_sigint_up
        if is_sigint_up:
            return

        '''Download Game Asset Index JSON file'''
        assetIndex = self.game().get('assetIndex')
        index_json_url = assetIndex.get('url')
        index_json_sha1 = assetIndex.get('sha1')
        index_json_path= os.path.join(self.config.assets_indexes_dir(), os.path.basename(index_json_url))
        if not checkFileExist(index_json_path,index_json_sha1):
            print("Download assetIndex JSON File")
            index_json_str = requests.get(index_json_url).text
            if isPy3:
                with open(index_json_path,'w',encoding='utf-8') as f:
                    f.write(index_json_str)
            else: 
                with open(index_json_path,'w') as f:
                    f.write(index_json_str.encode('utf-8'))               
        else:
            print("assetIndex JSON File have been downloaded")

    def downloadAssetObjects(self):
        global is_sigint_up
        if is_sigint_up:
            return

        '''Download Game Asset Objects'''
        objects = self.assets().get('objects')
        total = len(objects)

        errorMsg = []
        with progressbar.ProgressBar(max_value=total, prefix='assets objects: ') as bar:
            index = 0
            for (name,object) in objects.items():
                index = index + 1
                outInfo = '%d/%d(%s)' % (index, total, name)
                hash = object.get('hash')
                url = Mojang.assets_objects_url(hash)
                object_dir = self.config.assets_objects_dir(hash)
                object_filePath = os.path.join(object_dir,os.path.basename(url))
                if not checkFileExist(object_filePath, hash):
                    try:
                        self.download(url,object_dir)
                    except:
                        errorMsg.append(outInfo + "FAILED!")
                bar.update(index)
        
        if(len(errorMsg) > 0):
            print('\n'.join(errorMsg))
# Library

    def donwloadLibraries(self):

        global is_sigint_up
        if is_sigint_up:
            return

        ''' download libraries'''
        libs = self.game().get('libraries')
        total = len(libs)

        errorMsg = []
        with progressbar.ProgressBar(max_value=total, prefix='libraries: ') as bar:
            index = 0
            for lib in libs: 
                libName = lib.get('name')
                downloads = lib.get('downloads')

                rules = lib.get('rules')
                if None != rules:
                    for rule in rules:
                        if None != rule:
                            if rule.get('action') == 'disallow':
                                if rule.get('os').get('name') == platformType():
                                    errorMsg.append(libName + 'is disallowed')
                                    continue

                libPath = None
                url = None
                nativeKey = 'natives-'+ platformType()
                if 'natives' in lib:
                    platform = lib.get('natives').get(platformType())
                    if platform == None:
                        errorMsg.append('Error: no platform jar - ' +  libName)
                        continue
                    else:
                        libPath = downloads.get('classifiers').get(platform).get('path')
                        url = downloads.get('classifiers').get(platform).get('url')
                        sha1 = downloads.get('classifiers').get(platform).get('sha1')
                        nativeFilePath = os.path.join(self.config.client_native_dir(),os.path.basename(url))
                        if not checkFileExist(nativeFilePath,sha1):
                            self.download(url,self.config.client_native_dir())
                        
                else:
                    classifiers = downloads.get('classifiers')
                    if classifiers and nativeKey in downloads.get('classifiers'):
                        url = downloads.get('classifiers').get(nativeKey).get('url')
                        sha1 = downloads.get('classifiers').get(platform).get('sha1')
                        nativeFilePath = os.path.join(self.config.client_native_dir(),os.path.basename(url))
                        if not checkFileExist(nativeFilePath,sha1):
                            self.download(url,self.config.client_native_dir())
                    
                    libPath = downloads.get('artifact').get('path')
                    url = downloads.get('artifact').get('url')
                    sha1 = downloads.get('artifact').get('sha1')
                    fileDir = self.config.client_library_dir(libPath)
                    filePath=os.path.join(fileDir,os.path.basename(url))
                    if not checkFileExist(filePath,sha1):
                        self.download(url,fileDir)

                index = index + 1
                bar.update(index)
        if(len(errorMsg) > 0):
            print('\n'.join(errorMsg))
# Client

    def downloadClient(self):
        global is_sigint_up
        if is_sigint_up:
            return  

        '''Download Client Jar File'''
        client = self.game().get('downloads').get('client')
        clientUrl = client.get('url')
        sha1 = client.get('sha1')
        clientJARFilePath = os.path.join(self.config.versionDir(), os.path.basename(clientUrl))
        if not checkFileExist(clientJARFilePath, sha1):
            print('Downloading the client jar file ...')
            self.download(clientUrl,self.config.versionDir())
            print("Client Download Completed!")
        else:
            print("Client Jar File have been downloaded")

    def gameArguments(self, user, resolution):

        mainCls = self.game().get('mainClass')
        classPathList = self.javaClassPathList()
        sep = ';' if platformType() == 'windows' else ':'
        classPath = sep.join(classPathList)
        (res_width, res_height) = resolution

        configuration = {
            # for game args
            "auth_player_name" : user,
            "version_name" : self.game().get('id'),
            "game_directory" : self.config.GAME_ROOT_DIR,
            "assets_root" : self.config.GAME_ASSET_DIR,
            "assets_index_name" : self.game().get('assets'),
            "auth_uuid" : ''.join(str(uuid.uuid1()).split('-')),
            "auth_access_token" : ''.join(str(uuid.uuid1()).split('-')),
            "user_type" : "Legacy",
            "version_type" : "OrzMC",
            "resolution_width":res_width if res_width else "",
            "resolution_height":res_height if res_height else "",
            # for jvm args
            "natives_directory": self.config.client_native_dir(),
            "launcher_name": "OrzMC",
            "launcher_version": '1.0',
            "classpath": classPath,

        }

        arguments = [os.popen('which java').read().strip() if platformType() != 'windows' else 'javaw ']

        jvmArgs = self.game().get('arguments').get('jvm')
        argPattern = r'\$\{(.*)\}'
        for arg in jvmArgs:
            if isinstance(arg, str) or not isPy3 and isinstance(arg, unicode):
                value_placeholder = re.search(argPattern,arg)
                if value_placeholder:
                    argValue = configuration.get(value_placeholder.group(1))
                    argStr = re.sub(argPattern,argValue,arg)
                    arguments.append(argStr)
                else:
                    arguments.append(arg)            
            elif isinstance(arg, dict):
                isValid = False
                rules = arg.get('rules')
                for rule in rules:
                    if rule.get('os').get('name') == platformType() and rule.get('action') == 'allow':
                        isValid = True
                        break
                if isValid:
                    arguments.extend(arg.get('value'))

        arguments.append(mainCls)

        gameArgs = self.game().get('arguments').get('game')
        for arg in gameArgs:
            if isinstance(arg, str) or not isPy3 and isinstance(arg, unicode):
                value_placeholder = re.search(argPattern,arg)
                if value_placeholder:
                    argValue = configuration.get(value_placeholder.group(1))
                    argStr = re.sub(argPattern,argValue,arg)
                    arguments.append(argStr)
                else:
                    arguments.append(arg)
            elif isinstance(arg, dict):
                for rule in arg.get('rules'):
                    if rule.get('action') == 'allow':
                        if rule.get('features').get('is_demo_user'):
                            if user ==  None:
                                arguments.append(arg.get('value'))
                        if rule.get('features').get('has_custom_resolution'):
                            value = arg.get('value')
                            for arg in value:
                                if isinstance(arg, str):
                                    value_placeholder = re.search(argPattern,arg)
                                    if value_placeholder:
                                        argValue = configuration.get(value_placeholder.group(1))
                                        argStr = re.sub(argPattern,argValue,arg)
                                        arguments.append(argStr)
                                    else:
                                        arguments.append(arg)  


        arguments = ' '.join(arguments)
        return arguments

    def startCient(self, user, resolution = (None,None)):
        global is_sigint_up
        if is_sigint_up:
            return

        cmd = self.gameArguments(user,resolution)
        backgroundCmd = 'start ' + cmd if platformType == 'windows' else cmd + ' &'
        os.system(backgroundCmd)

# Server
    def serverJARFilePath(self):
            server = self.game().get('downloads').get('server')
            serverUrl = server.get('url')
            sha1 = server.get('sha1')
            serverJARFilePath = os.path.join(self.config.versionDir(), os.path.basename(serverUrl))
            return (serverJARFilePath, serverUrl, sha1)

    def downloadServer(self):
        '''Download Server Jar File'''
        (serverJARFilePath, serverUrl, sha1) = self.serverJARFilePath()
        if not checkFileExist(serverJARFilePath, sha1):
            print('Downloading the server jar file ...')
            self.download(serverUrl,self.config.server_jar_path())
            print("Server Download Completed!")
        else:
            print("Server Jar File have been downloaded")

    def deployServer(self, mem_start='512M', mem_max="1024M"):
        '''deploy minecraft server'''
        if self.config.isSpigot and not os.path.exists(self.config.server_spigot_jar_path()):
            self.buildSpigotServer()
        (serverJARFilePath, _, _) = self.serverJARFilePath()
        jarFilePath = self.config.server_spigot_jar_path() if self.config.isSpigot else serverJARFilePath
        self.startServer(self.startCommand(mem_start, mem_max, jarFilePath, jvm_opts=' -XX:+UseConcMarkSweepGC'))

    def startCommand(self, mem_s, mem_x, serverJARFilePath, jvm_opts = ''):
        '''construct server start command'''
        cmd = 'java' + jvm_opts + ' -Xms' + mem_s + ' -Xmx' + mem_x + ' -jar ' + serverJARFilePath + ' nogui'
        return cmd

    def checkEULA(self):
        return os.path.exists(self.config.eula_path())

    def startServer(self, cmd):
        '''启动minecraft服务器'''
        
        os.chdir(self.config.server_deploy_path())

        # 如果没有eula.txt文件，则启动服务器生成
        if not self.checkEULA():
            os.system(cmd)

        # 同意eula
        with io.open(self.config.eula_path(), 'r', encoding = 'utf-8') as f:
            eula = f.read()
            checkEULA = eula.replace('false', 'true')
        with io.open(self.config.eula_path(), 'w', encoding = 'utf-8') as f:
            f.write(checkEULA)
        
        # 启动服务
        os.system(cmd)

        # 设置服务器属性为离线模式
        with io.open(self.config.properties_path(), 'r', encoding = 'utf-8') as f:
            properties = f.read()
            if 'online-mode=false' in properties:
                offline_properties = ''
            else:
                offline_properties = properties.replace('online-mode=true', 'online-mode=false')
        with io.open(self.config.properties_path(), 'w', encoding = 'utf-8') as f:
            if len(offline_properties) > 0:
                f.write(offline_properties)
                print(ColorString.confirm('Setting the server to offline mode, next launch this setting take effect!!!'))

    def buildSpigotServer(self):
        '''构建SpigotServer'''
        version = self.config.version
        spigot = Spigot(version)
        print(ColorString.warn('Start download the spigot build tool jar file...'))
        self.download(spigot.build_tool_jar, self.config.server_deploy_build_path())
        print(ColorString.confirm('Build tool jar download completed!!!'))
        buildToolJarName = os.path.basename(spigot.build_tool_jar)
        buildToolJarPath = os.path.join(self.config.server_deploy_build_path(), buildToolJarName)
        versionCmd = ' --rev ' + version if len(version) > 0 else 'lastest'
        linuxCmd = 'git config --global --unset core.autocrlf; java -jar ' + buildToolJarPath + versionCmd
        macCmd = 'export MAVEN_OPTS="-Xmx2G"; java -Xmx2G -jar ' + buildToolJarPath + versionCmd
        windowCmd = 'java -jar ' + buildToolJarPath + versionCmd
        cmd = macCmd if platformType() == 'osx' else linuxCmd if platformType() == 'linux' else windowCmd
        print(ColorString.warn('Start build the server jar file with build tool...'))
        os.chdir(self.config.server_deploy_build_path())
        os.system(cmd)
        print(ColorString.confirm('Completed! And the spigot built server file generated!'))
        shutil.move(self.config.server_spigot_jar_path(isInBuildDir=True), self.config.server_spigot_jar_path())
        shutil.move(self.config.server_craftbukkit_jar_path(isInBuildDir=True), self.config.server_craftbukkit_jar_path())
        os.chdir(self.config.server_deploy_path())
        shutil.rmtree(self.config.server_deploy_build_path())

        


# -*- coding: utf8 -*-

import hashlib
import os
import sys
import platform

isPy3 = (sys.version_info.major >= 3)

def checkFileExist(filePath, hash):
    if not os.path.exists(filePath):
        return False
    
    with open(filePath, 'rb') as f:
        computeHash = hashlib.sha1(f.read()).hexdigest()
        ret = (computeHash == hash)
        if not ret:
            print('sha1 check failed: ' + filePath)
        return ret

def makedirs(path):
    if not os.path.exists(path):
        os.makedirs(path)


def platformType():
    '''OS type'''
    system = {
        'Linux':'linux',
        'Darwin':'osx',
        'Windows': 'windows'
    }
    return system[platform.system()]

def hint(msg):
    if isPy3:
        return input(msg)
    else:
        return raw_input(msg)

class ColorString:

    FG_BLACK = 30
    FG_RED = 31
    FG_GREEN = 32
    FG_YELLOW = 33
    FG_BLUE = 34
    FG_PURPLE = 35
    FG_CYAN = 36
    FG_WHITE = 37

    BG_BLACK = 40
    BG_RED = 41
    BG_GREEN = 42
    BG_YELLOW = 43
    BG_BLUE = 44
    BG_PURPLE = 45
    BG_CYAN = 46
    BG_WHITE = 47

    DEFUALT = 0
    HIGHLIGHT = 1
    UNDERLINE = 2
    TWINKLE = 3
    INVERSE = 4
    INVISIBLE = 5

    @classmethod
    def string(cls, str, fg = FG_WHITE, bg = BG_BLACK, displayMode = DEFUALT):
        return '\033[%s;%s;%sm%s\033[0m' % (displayMode, fg, bg, str)

    @classmethod
    def warn(cls, str):
        return ColorString.string(str, ColorString.FG_YELLOW, displayMode=ColorString.HIGHLIGHT)

    @classmethod
    def confirm(cls, str):
        return ColorString.string(str, ColorString.FG_GREEN, displayMode=ColorString.HIGHLIGHT)

    @classmethod
    def error(cls, str):
        return ColorString.string(str, ColorString.FG_RED, displayMode=ColorString.HIGHLIGHT)

    @classmethod
    def hint(cls, str):
        return ColorString.string(str, ColorString.FG_RED, displayMode=ColorString.HIGHLIGHT)


class Spigot:

    build_tool_jar = 'https://hub.spigotmc.org/jenkins/job/BuildTools/lastSuccessfulBuild/artifact/target/BuildTools.jar'
    
    def __init__(self, version):
        self.server_version = version

